/*
 * include/asm-m68knommu/linux_logo.h: This is a linux logo
 *                                 to be displayed on boot.
 */
 
#include <linux/config.h>
#include <linux/init.h>
#include <linux/version.h>

#define linux_logo_banner "Linux/m68knommu version " UTS_RELEASE

#include <linux/linux_logo.h>
