/*
 *  linux/include/asm-frionommu/hdreg.h
 *
 *  Copyright (C) 1994-1996  Linus Torvalds & authors
 *
 *  Chabges made by Lineo Inc	May 2001
 */

#ifndef _FRIO_HDREG_H
#define _FRIO_HDREG_H

typedef unsigned char * ide_ioreg_t;

#endif /* _FRIO_HDREG_H */
