/*
 * Rev:$Id: blackfin.h,v 1.1 2003/11/04 14:47:41 bas Exp $
 *
 * Copyright (C) 2003 Bas Vermeulen/Buyways B.V. <bf533@buyways.nl>
 *
 * Created by Buyways B.V. for Astent <info@astent.nl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#ifndef _BLACKFIN_H_
#define _BLACKFIN_H_
#ifdef CONFIG_PUB
#include <asm/board/pub.h>
#endif
#ifdef CONFIG_HAWK
#include <asm/board/hawk.h>
#endif
#ifdef CONFIG_EAGLE
#include <asm/board/eagle.h>
#endif
#ifdef CONFIG_BLACKFIN_EZKIT
#include <asm/board/ezkit.h>
#endif
#endif /* _BLACKFIN_H_ */
