/*
 * asm/arch-armnommu/arch-p52/dma.h:
 * Mindspeed 2001
 */

#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H


#define MAX_DMA_ADDRESS		0x01000000
#define MAX_DMA_CHANNELS        13
#define arch_dma_init(dma_chan)

#endif /* _ASM_ARCH_DMA_H */
