/*
 *  Copyright (C) 1994-1996  Linus Torvalds & authors
 *  Copyright (C) 2002  Wentau Xu (www.microtronix.com)
 */

#ifndef __NIOS_HDREG_H
#define __NIOS_HDREG_H

typedef unsigned long ide_ioreg_t;

#endif /* __NIOS_HDREG_H */
