/*
 * linux/include/asm-arm/arch-sa1100/simputer.h
 *
 * Created 2001/07/11 by Vivek K S <ksvivek@picopeta.com>
 * Modified from Nicolas Pitre's assabet.h
 *
 *
 * This file contains the hardware specific definitions for simputer
 * Only include this file from SA1100-specific files.
 *
 */
#ifndef __ASM_ARCH_SIMPUTER_H
#define __ASM_ARCH_SIMPUTER_H

#define GPIO_UCB1300_IRQ	GPIO_GPIO (5)	/* UCB GPIO and touchscreen */
#define IRQ_GPIO_UCB1300_IRQ IRQ_GPIO5

#endif
