#ifndef _HYPERSTONE_NOMMU_SIGCONTEXT_H
#define _HYPERSTONE_NOMMU_SIGCONTEXT_H

/*
 * Signal context structure - contains all info to do with the state
 * before the signal handler was invoked.  Note: only add new entries
 * to the end of the structure.
 */
struct sigcontext {
};


#endif /* _HYPERSTONE_NOMMU_SIGCONTEXT_H */
