#ifndef __HYPERSTONE_BUGS_H
#define __HYPERSTONE_BUGS_H
/*
 * This is included by init/main.c to check for architecture-dependent bugs.
 *
 * Needs:
 *	void check_bugs(void);
 */

static void check_bugs(void)
{
}
#endif 
