/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * Copyright (C) 1994, 1995, 1996, 1999 by Ralf Baechle
 * Copyright (C) 1999 Silicon Graphics, Inc.
 */
#ifndef _ASM_MIPS_BOARDS_IO_H
#define _ASM_MIPS_BOARDS_IO_H

#include <asm/addrspace.h>

#define IO_SPACE_BASE K1BASE

#define IO_SPACE_LIMIT 0x0fffffff

#endif /* _ASM_MIPS_BOARDS_IO_H */
