/*
 * include/asm-v850/v850e.h -- V850E CPU
 *
 *  Copyright (C) 2001,02,03  NEC Electronics Corporation
 *  Copyright (C) 2001,02,03  Miles Bader <miles@gnu.org>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 * Written by Miles Bader <miles@gnu.org>
 */

#ifndef __V850_V850E_H__
#define __V850_V850E_H__

#include <asm/v850e_intc.h>

#define CPU_ARCH "v850e"

#endif /* __V850_V850E_H__ */
