/*
 * uclinux/include/asm-armnommu/arch-S3C3410/pcmcia.h
 *
 * (just a placeholder, not needed in our machine)
 *
 * 2003 Thomas Eschenbacher <thomas.eschenbacher@gmx.de>
 *
 */
#ifdef CONFIG_REVISIT
#error Do we need PCMCIA support?
#endif
