/*
 * linux/include/asm/arch-S3C3410/serial.h
 *
 * (C) 2003 sympat GmbH
 * by Thomas Eschenbacher <thomas.eschenbacher@sympat.de>
 *
 */
#ifndef __ASM_ARCH_SERIAL_H
#define __ASM_ARCH_SERIAL_H

/* 
 * nothing needed here, all definitions are in 
 * drivers/char/serial_s3c3410.h
 */

#endif /* __ASM_ARCH_SERIAL_H */
