/*
 *      include/asm-mips64/dec/io.h
 *
 *      Copyright (C) 2002  Maciej W. Rozycki
 *
 *      This program is free software; you can redistribute it and/or
 *      modify it under the terms of the GNU General Public License
 *      as published by the Free Software Foundation; either version
 *      2 of the License, or (at your option) any later version.
 */
#ifndef __ASM_MIPS64_DEC_IO_H
#define __ASM_MIPS64_DEC_IO_H

#include <asm/addrspace.h>

#define IO_SPACE_BASE K1BASE

#define IO_SPACE_LIMIT 0xffffffff

#endif /* __ASM_MIPS64_DEC_IO_H */
