/*
 * asm/arch-armnommu/arch-s3c4/dma.h:
 * OZH, 2001
 */

#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#define MAX_DMA_ADDRESS		0x03000000
#define MAX_DMA_CHANNELS        2

#endif /* _ASM_ARCH_DMA_H */
