/*
 * uclinux/include/asm-armnommu/arch-S3C44B0X/pcmcia.h
 *
 * (just a placeholder, not needed in our machine)
 *
 * Copyright (C) 2003 Thomas Eschenbacher <eschenbacher@sympat.de>
 *
 */
#ifdef CONFIG_REVISIT
#error Do we need PCMCIA support?
#endif
