/*
	This is just a fragment of the original "traps.h" file,
	enough to get the one symbol defined for "traps.c".
*/
#ifndef _NIOS_TRAPS_H
#define _NIOS_TRAPS_H

/* Now the Software Traps... */
#define SP_TRAP_SBPT    0x81         /* Software Breakpoint */

#endif /* !(_NIOS_TRAPS_H) */
