/*
 * uclinux/include/asm-armnommu/arch-S3C44B0X/dma.h:
 *
 * Copyright (C) 2003 Thomas Eschenbacher <eschenbacher@sympat.de>
 *
 */

#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

#define MAX_DMA_ADDRESS   0x03000000
#define MAX_DMA_CHANNELS  2

#endif /* _ASM_ARCH_DMA_H */
