/*
 *  linux/include/asm-niosnommu/byteorder.h
 */
#ifndef __ASM_NIOS_BYTEORDER_H
#define __ASM_NIOS_BYTEORDER_H


#include <asm/types.h>

//vic FIXME need some swab defines ...

//vic#if !defined(__STRICT_ANSI__) || defined(__KERNEL__)
//vic#  define __BYTEORDER_HAS_U64__
//vic#  define __SWAB_64_THRU_32__
//vic#endif

#include <linux/byteorder/little_endian.h>

#endif

