/*
 * uclinux/include/asm-armnommu/arch-S3C3410/a.out.h
 *
 * (just a placeholder, not needed in our machine)
 *
 * 2003 Thomas Eschenbacher <thomas.eschenbacher@gmx.de>
 *
 */
#ifndef __ARCH_ASM_PARAM_H__
#define __ARCH_ASM_PARAM_H__

/* nothing for now */

#endif /* __ARCH_ASM_PARAM_H__ */
