/*
 * include/asm-frionommu/linux_logo.h: This is a linux logo
 *                                 to be displayed on boot.
 */
 
#include <linux/config.h>
#include <linux/init.h>
#include <linux/version.h>

#define linux_logo_banner "Linux/frionommu version " UTS_RELEASE

