#ifndef __ALPHA_DELAY_H
#define __ALPHA_DELAY_H

extern void __delay(int loops);
extern void ndelay(unsigned long nsecs);
extern void udelay(unsigned long usecs);

#endif /* defined(__ALPHA_DELAY_H) */
