/*
 * uclinux/include/asm-armnommu/arch-S3C44B0X/a.out.h
 *
 * (just a placeholder, not needed in our machine)
 *
 * Copyright (C) 2003 Thomas Eschenbacher <eschenbacher@sympat.de>
 *
 */
#ifndef __ASM_ARCH_A_OUT_H
#define __ASM_ARCH_A_OUT_H

#ifdef __KERNEL__
#endif

#endif

