/*
 * uclinux/include/asm-armnommu/arch-S3C3410/timex.h
 *
 * (just a placeholder, not needed in our machine)
 *
 * 2003 Thomas Eschenbacher <thomas.eschenbacher@gmx.de>
 *
 */
#ifndef __ASM_ARCH_TIMEX_H__
#define __ASM_ARCH_TIMEX_H__

#endif /* __ASM_ARCH_TIMEX_H__ */
