/* References to section boundaries */

#ifndef __SPARC64_SECTIONS_H
#define __SPARC64_SECTIONS_H

extern char __init_begin[], __init_end[];

extern char _start[], _end[];

extern char etext[], edata[];

#endif /* !(__SPARC64_SECTIONS_H) */
