/*
 * linux/include/asm-arm/arch-sa1100/adsbitsy.h
 *
 * Created 7/3/01 by Woojung <whuh@applieddata.net>
 *
 * This file contains the hardware specific definitions for the
 * ADS Bitsy Board
 */

#ifndef __ASM_ARCH_HARDWARE_H
#error "include <asm/hardware.h> instead"
#endif

#define ADSBITSY_SA1111_BASE	(0x18000000)
