/*
 *	include/asm-mips/irq_cpu.h
 *
 *	MIPS CPU interrupt definitions.
 *
 *	Copyright (C) 2002  Maciej W. Rozycki
 *
 *	This program is free software; you can redistribute it and/or
 *	modify it under the terms of the GNU General Public License
 *	as published by the Free Software Foundation; either version
 *	2 of the License, or (at your option) any later version.
 */
#ifndef __ASM_MIPS_IRQ_CPU_H
#define __ASM_MIPS_IRQ_CPU_H

extern void mips_cpu_irq_init(int irq_base);

#endif /* __ASM_MIPS_IRQ_CPU_H */
