#ifndef _PPC64_SHMPARAM_H
#define _PPC64_SHMPARAM_H

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

#define	SHMLBA PAGE_SIZE		 /* attach addr a multiple of this */

#endif /* _PPC64_SHMPARAM_H */
