/*
 * linux/include/asm-cris/namei.h
 *
 * Included from linux/fs/namei.c
 */

#ifndef __CRIS_NAMEI_H
#define __CRIS_NAMEI_H

/* used to find file-system prefixes for doing emulations
 * see for example asm-sparc/namei.h
 * we don't use it...
 */

#define __emul_prefix() NULL

#endif /* __CRIS_NAMEI_H */
