#ifndef _HYPERSTONE_NOMMU_SHMPARAM_H
#define _HYPERSTONE_NOMMU_SHMPARAM_H

#if 0 /* Out for the time being */
#define	SHMLBA PAGE_SIZE		 /* attach addr a multiple of this */
#endif

#endif /* _HYPERSTONE_NOMMU_SHMPARAM_H */
