/*
 * linux/include/asm-v850/namei.h
 *
 * Included from linux/fs/namei.c
 */

#ifndef __V850_NAMEI_H__
#define __V850_NAMEI_H__

/* This dummy routine maybe changed to something useful
 * for /usr/gnemul/ emulation stuff.
 * Look at asm-sparc/namei.h for details.
 */

#define __emul_prefix() NULL

#endif /* __V850_NAMEI_H__ */
