/*
 * timex.h:
 * 2001 Mac Wang <mac@os.nctu.edu.tw> 
 */
#ifndef __ASM_ARCH_TIMEX_H__
#define __ASM_ARCH_TIMEX_H__

#include <asm/hardware.h>

#define CLOCK_TICK_RATE	   (((fMCLK_MHz/100))*2)

#endif /* __ASM_ARCH_TIMEX_H__ */
