/*
 *  linux/include/asm-arm/linux_logo.h
 *
 *  Copyright (C) 1998 Russell King
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  Linux console driver logo definitions for ARM
 */

#include <linux/init.h>
#include <linux/version.h>

#define linux_logo_banner "ARM Linux version " UTS_RELEASE

#include <linux/linux_logo.h>

