#ifndef __ARCH_NIOSNOMMU_CACHE_H
#define __ARCH_NIOSNOMMU_CACHE_H

/* bytes per L1 cache line */
#define        L1_CACHE_BYTES  16	/* this need to be at least 1 */


#define __cacheline_aligned
#define ____cacheline_aligned

#endif
