/*
 * linux/include/asm-arm/arch-nexuspci/dma.h
 *
 * Architecture DMA routines
 *
 * Copyright (C) 1998, 1999 Philip Blundell
 */

/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 */

/*
 * This is the maximum DMA address that can be DMAd to.
 */
#define MAX_DMA_ADDRESS		0xffffffff
#define MAX_DMA_CHANNELS	0
