#ifndef __ASM_SH64_HW_IRQ_H
#define __ASM_SH64_HW_IRQ_H

/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 * include/asm-sh64/hw_irq.h
 *
 * Copyright (C) 2000, 2001  Paolo Alberelli
 *
 */
static __inline__ void hw_resend_irq(struct hw_interrupt_type *h, unsigned int i) { /* Nothing to do */ }

#endif /* __ASM_SH64_HW_IRQ_H */
