#ifndef _INCLUDE_LART_H
#define _INCLUDE_LART_H

#define LART_GPIO_ETH0			GPIO_GPIO0
#define LART_IRQ_ETH0			IRQ_GPIO0

#define LART_GPIO_IDE			GPIO_GPIO1
#define LART_IRQ_IDE			IRQ_GPIO1

#define LART_GPIO_UCB1200		GPIO_GPIO18
#define LART_IRQ_UCB1200		IRQ_GPIO18

#endif
