/*
 * Copyright (c) 2003, Bernard Leach (leachbj@bouncycastle.org)
 */

#ifndef __ASM_ARCH_DMA_H
#define __ASM_ARCH_DMA_H

/* we don't have any DMA */
#define MAX_DMA_ADDRESS		0xffffffff
#define MAX_DMA_CHANNELS	0

#endif

