/*
 * uclinux/include/asm-armnommu/arch-S3C44B0X/timex.h
 *
 * (just a placeholder, not needed in our machine)
 *
 * Copyright (C) 2003 Thomas Eschenbacher <eschenbacher@sympat.de>
 *
 */
#ifndef __ASM_ARCH_TIMEX_H__
#define __ASM_ARCH_TIMEX_H__

#endif /* __ASM_ARCH_TIMEX_H__ */
