/*
 * include/asm-microblaze/microblaze_system_uart.h -- system UART required 
 * 	for booting a microblaze uClinux system
 *
 *  Copyright (C) 2003  John Williams <jwilliams@itee.uq.edu.au>
 *  Copyright (C) 2001  NEC Corporation
 *  Copyright (C) 2001  Miles Bader <miles@gnu.org>
 *
 * This file is subject to the terms and conditions of the GNU General
 * Public License.  See the file COPYING in the main directory of this
 * archive for more details.
 *
 * Written by Miles Bader <miles@gnu.org>
 * Microblaze port by John Williams
 */

/* 
 * Note this is all place-holder stuff grabbed from v850 code.  
 * Doesn't do anything yet.
 */

#ifndef __MICROBLAZE_MICROBLAZE_SYSTEM_UART_H__
#define __MICROBLAZE_MICROBLAZE_SYSTEM_UART_H__

#include <asm/types.h>
#include <asm/machdep.h>	/* Pick up chip-specific defs.  */

#endif /* __MICROBLAZE_MICROBLAZE_SYSTEM_UART_H__ */
