/*
 * This file is subject to the terms and conditions of the GNU General Public
 * License.  See the file "COPYING" in the main directory of this archive
 * for more details.
 *
 *  This file contains the MIPS architecture specific IDE code.
 *
 *  Copyright (C) 1994-1996  Linus Torvalds & authors
 *  Copyright (C) 2001 Ralf Baechle
 */

#ifndef _ASM_HDREG_H
#define _ASM_HDREG_H

typedef unsigned long ide_ioreg_t;

#endif /* _ASM_HDREG_H */

